import os

import click

from minimize.util import get_py_files, file_processed_message
from minimize.consts import CLI_LINE_LENGTH


@click.command()
@click.argument("globs", nargs=-1)
def minimize(globs):
    globs = globs or ("./**/*",)
    files = get_py_files(globs)
    print("=" * CLI_LINE_LENGTH)
    for file in files:
        size_bytes = os.path.getsize(file)
        print(file_processed_message(file, size_bytes, size_bytes, CLI_LINE_LENGTH))
    print("-" * CLI_LINE_LENGTH)
    print("All done! \N{left-pointing magnifying glass}")
    print("=" * CLI_LINE_LENGTH)



import glob


def unique(_list):
    return list(set(_list))


def get_py_files(globs):
    py_files = [file for g in globs for file in glob.glob(g) if file.endswith(".py")]
    return unique(py_files)


def file_processed_message(filename, original_size, reduced_size, line_length):
    percent = get_percent_reduced(original_size, reduced_size)
    filename_string = f"Minimized {filename}"
    info_string = f"{original_size} bytes -> {reduced_size} bytes ({percent:.2f}%)"
    padding = line_length - len(filename_string)
    return f"{filename_string}{info_string:>{padding}}"


def get_percent_reduced(original, reduced):
    try:
        percent_reduced = 100 - (reduced / original) * 100
    except ZeroDivisionError:
        percent_reduced = 0
    return percent_reduced



from setuptools import setup, find_packages
from pathlib import Path

version = "0.0.4"
here = Path(__file__).parent.resolve()

with open(here.joinpath("README.md"), encoding="utf-8") as f:
    long_description = f.read()

with open(here.joinpath("requirements.txt"), encoding="utf-8") as f:
    requirements = f.readlines()

setup(
    name="python-minimize",
    version=version,
    description="Reduce the size of MicroPython projects",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/TSNobleSoftware/minimize",
    author="Tom Noble",
    author_email="t.s.noble@outlook.com",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: Implementation :: MicroPython",
    ],
    keywords=["python", "micropython", "cli"],
    packages=find_packages(),
    entry_points={"console_scripts": ["minimize=minimize.cli:minimize"]},
    python_requires=">=3.8",
    install_requires=[requirements],
    project_urls={
        "Bug Reports": "https://github.com/TSNobleSoftware/minimize/issues",
        "Source": "https://github.com/TSNobleSoftware/minimize",
    }
)
